/************************************************************/
/*                                                          */
/*             CONFIGURATION LIBRARY HEADER FILE            */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODULE     : CfgOut.h                               */
/*      AUTHOR     : F. BELLACHIA                           */
/*      LABORATORY : L.A.P.P.                               */
/*      VERSION    : v0r0                                   */
/*      DATE       : 13/06/01                               */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODIFICATION:                                       */
/*                                                          */
/************************************************************/
/*                                                          */
/*                                                          */
/************************************************************/
/* Copyright (c) 2001 LAPP Annecy, UMR-IN2P3-CNRS (France). */
/*                                                          */
/* Redistribution and use in source and binary forms, with  */
/* or without modification, are permitted provided that the */
/* following conditions are met:                            */
/* 1. Redistributions of source code must retain the above  */
/*    copyright notice, this list of conditions and the     */
/*    following disclaimer.                                 */
/* 2. Redistributions in binary form must reproduce the     */
/*    above copyright notice, this list of conditions and   */
/*    the following disclaimer in the documentation and/or  */
/*    other materials provided with the distribution.       */
/* 3. All advertising materials mentioning features or use  */
/*    of this software must display the following           */
/*    acknowledgement:                                      */
/*      This product includes software developed by the     */
/*      Computer Application Development Group at LAPP      */
/*      Annecy  (Laboratoire d'Annecy le vieux de Physique  */
/*      des Particules - UMR-IN2P3-CNRS).                   */
/* 4. Neither the name of the Institute nor of the          */
/*    Laboratory may be used to endorse or promote products */
/*    derived from this software without specific prior     */
/*    written permission.                                   */
/*                                                          */
/* THIS SOFTWARE IS PROVIDED BY THE LAPP AND CONTRIBUTORS   */
/* ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,         */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL THE LAPP OR CONTRIBUTORS   */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER   */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY */
/* OF SUCH DAMAGE.                                          */
/************************************************************/

#include <CfgHeader.h>
#include <CmMessage.h>
#include <CfgParse.h>

#ifndef CfgOut_h
#define CfgOut_h

#ifdef __cplusplus
extern "C" { 
#endif /* __cplusplus */

typedef enum CfgOutStatus {
  CfgOutNone = 0,
  CfgOutUnknownByNS,
  CfgOutConnectInProgress,
  CfgOutHostUnreachable,
  CfgOutHostAlive,
  CfgOutMsgSending,
  CfgOutMsgNew
} CfgOutStatus_t;

typedef enum CfgOutRequest {
  CfgOutStatic = 0,
  CfgOutDynamic,
  CfgOutOnRequest
} CfgOutRequest_t;

#define CFGOUT_TCP_TMO    -1.
#define CFGOUT_INPROGRESS CFG_CONTINUE
#define CFGOUT_CSM_ADD_TYPE  "CFGOUT_CSM_ADD"
#define CFGOUT_CSM_RM_TYPE   "CFGOUT_CSM_RM"
#define CFGOUT_GRP_ADD_TYPE  "CFGOUT_GRP_ADD"
#define CFGOUT_GRP_RM_TYPE   "CFGOUT_GRP_RM"

typedef int (*CfgOutPrinter_t)(char* format, va_list args);
typedef int (*CfgOutBuildMsg_t)(CmMessage message, void *arg, char *consumer,
                                char *group);

extern void CfgOutCmEnd(void);
extern void CfgOutCmInit(void);
extern int CfgOutDomainGroupAdd(char *domain, char *group);
extern int CfgOutDomainGroupRemove(char *domain, char *group);
extern CfgOutPrinter_t CfgOutInstallPrinter(CfgOutPrinter_t printer);
extern int CfgOutGroupConsumerAdd(char *group, char *name, int retry,
                                  int priority, CfgOutRequest_t request,
                                  char *owner, char *host);
extern int CfgOutGroupConsumerDataAdd(char *group, char *name, Ulong data);
extern int CfgOutGroupConsumerPost(char *group, char *name,
                                   CfgOutBuildMsg_t buildMsg, void *arg);
extern int CfgOutGroupConsumerRemove(char *group, char *name);
extern int CfgOutGroupConsumerRequest(char *group, char *name,
                                      CfgOutBuildMsg_t buildMsg, void *arg,
                                      double timeout);
extern CfgOutStatus_t CfgOutGroupConsumerStatusGet(char *group, char *name);
extern int CfgOutGroupDebugSet(char *group, int debug);
extern char *CfgOutGroupDefaultGet(void);
extern void CfgOutGroupDefaultSet(char *group);
extern void CfgOutGroupFree(char *group);
extern CfgOutBuildMsg_t CfgOutGroupInstallBuilder(char *group,
                                                  CfgOutBuildMsg_t buildMsg,
                                                  void *arg);
extern int CfgOutGroupNew(char *group, CfgParse_t *cfgParse, int maxConsumer,
                          int maxGroup, int threshold, int debug);
extern int CfgOutGroupNsPeriodSet(char *group, int period);
extern int CfgOutGroupPost(char *group, CfgOutBuildMsg_t buildMsg, void *arg);
extern int CfgOutGroupStatusGet(char *group, char *name,
                                CfgOutStatus_t *status);
extern void CfgOutGroupThresholdSet(char *group, int threshold);
extern CfgOutBuildMsg_t CfgOutGroupUninstallBuilder(char *group);
extern int CfgOutParserInit(CfgParse_t *cfgParse);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* CfgOut_h */
